/*******************************************************************************

    Policyd.H
    $Id: Policyd.H,v 1.19.2.2 2010/03/09 06:26:21 qling Exp $

********************************************************************************/
#ifndef POLICYD_H
#define POLICYD_H

typedef enum 
{
    POLICYD_ERROR = -500, 
    POLICYD_BAD_PARAM,           // bad input parameter
    POLICYD_BAD_CONFIG,           // bad input parameter
    POLICYD_ENV_VAR_NOT_DEFINED, // can not get environment variable
    POLICYD_COMM_ERROR,         // communication errors 
    POLICYD_MAX_CONNECTIONS,    // exceed the connections limits
    POLICYD_TIMEOUT,               //query timeout
    POLICYD_RET_OK = 0
} POLICYD_RET_CODE ;

typedef enum
{
    POLICYD_ACTION_BLOCK = 0,
    POLICYD_ACTION_ALLOW,
    POLICYD_ACTION_NO_MATCH,
  	POLICYD_ACTION_UNAUTHORIZED,  //only for queryHttpPolicy() with user name
} POLICYD_ACTION ;

typedef enum
{
    AUTH_MODE_UNKNOWN,
    AUTH_MODE_LOCAL,
    AUTH_MODE_LDAP,
    AUTH_MODE_RADIUS
} AuthMode ;

typedef enum
{
    POLICY_TYPE_UNKNOWN,
    POLICY_TYPE_TRAFFIC, 
    POLICY_TYPE_URL,
    POLICY_TYPE_CATEGORY,
    POLICY_TYPE_APP,
    POLICY_TYPE_SAFE_SEARCH,
    POLICY_TYPE_FILENAME
} PolicyType ;

struct PolicyLogger
{
    int logLevel ;
    void (*writeLog) (const char *msg) ;
} ;
 
#ifdef __cplusplus 
extern "C"
{
#endif
// set logger function for policyd client
void setPolicyLogger(struct PolicyLogger* logger) ;

// create policyd's client instance
POLICYD_RET_CODE createPolicyClientInstance(void **ins) ;

// set timeout for policyd client, default timeout = 0s except logon (queryHttp with username)
void setPolicydClientTimeout(void *instance, int t_sec) ;

#define MAX_AUTHSVR_NUM 20
#define MAX_AUTHNAME_LEN 256
#define MAX_CATEGORY_NUM 256
#define MAX_MATCH_RULE_LEN 1024

struct PolicyAuthOutput
{
    char username[MAX_AUTHNAME_LEN];
    AuthMode   auth_mode;
    char server_alias[MAX_AUTHNAME_LEN];
    char serverlist[MAX_AUTHSVR_NUM][MAX_AUTHNAME_LEN];
};

struct PolicyAuthInput
{
    const char *username;
    const char *password;
    AuthMode   auth_mode;
    const char *server_alias;
    char          tryflag;          //0: donot try to use other domains; 1: try others
};
// 1 if username is not null or empty and server_alias is null or empty, 
//    policyd will try to login with a domain in turns of loc,ldap,radius
// 2 if server_alias is not null or empty and tryflag==1, 
//    policyd will try to login with server_alias, if failed, it will try to use other domains

struct PolicyAuthInfo
{
    struct PolicyAuthInput auth_in;
    struct PolicyAuthOutput auth_out;
};

#define MAX_GRUOPNAMES_LEN 1024
struct PolicyUserInfo
{
    char name[MAX_AUTHNAME_LEN];        //if name[0]=0, unauthenticated
    char type[10];                                     //LOC/LDAP/RADIUS
    char domain[MAX_AUTHNAME_LEN];
    char group[MAX_GRUOPNAMES_LEN];   //mulitnames separated by comma
    char langroup[MAX_AUTHNAME_LEN];  //if name[0]=0 unauthenticated, langroup may be not empty
};

// query user info and update user last seen time
POLICYD_RET_CODE queryUserInfo(
    void *instance,
    const char * ip,                             //IN: user's IP
    const char * ip2,                           //IN: STM: user's IP, if ip2=NULL, only check *ip; UTM: NULL: vpn user; others: non vpn user
    struct PolicyUserInfo *user_info   //OUT this IP's user info
                                                        //if user_info->name[0]='\0', this IP is unauthenticated(but may have langroup)
) ;

// query http policy
#define DONOT_USE_URLD 0xFFFFFFFF
POLICYD_RET_CODE queryHttpPolicy(
    void *instance,
    struct PolicyAuthInfo *auth_info,  //IN and OUT: it can be NULL if needn't
                                                        //if auth_info->auth_in.username='\0', it will return auth_out
    const char * url,                            //IN: URL that should be checked, if url=NULL, only authenticate
    const char * ip,                             //IN: user's IP
    const char * ip2,                           //IN: STM: user's IP, if ip2=NULL, only check *ip;UTM: NULL: non vpn user; others: vpn user
    const unsigned int  * cat_result,    //IN: category ids of this url
    unsigned int  cat_number,            //IN: the category number of cat_result;0xFFFFFFFF skip category rules
    char * match_rule,                      //OUT: matched rule: cat:<cat id>; url:<url pattern>; traffic:;
    							      //NOTE: cat:- uncategorized; cat:* all categories
    unsigned int match_rule_length,  //IN: the maximum length of matched rule
    POLICYD_ACTION *ret,                 //OUT: return action
    struct PolicyUserInfo *user_info   //OUT this IP's user info, it can be NULL if needn't
                                                        //if user_info->name[0]='\0', this IP is unauthenticated(but may have langroup)
) ;

// query safe search policy
POLICYD_RET_CODE querySafeSearchPolicy(
    void *instance,
    const char * safe_search_type,
    const char * ip,
    const char * ip2,                           //IN: STM:user's IP, if ip2=NULL, only check *ip; UTM: NULL: non vpn user; others: vpn user
    POLICYD_ACTION *ret,                 // OUT: return action
    struct PolicyUserInfo *user_info   //OUT this IP's user info
                                                        //if user_info->name[0]='\0', this IP is unauthenticated(but may have langroup)
) ;
    
// query app policy
POLICYD_RET_CODE queryAppPolicy(
    void *instance,
    const char * imp2p_type,
    const char * ip,
    const char * ip2,                            //IN: STM:user's IP, if ip2=NULL, only check *ip; UTM: NULL: non vpn user; others: vpn user
    POLICYD_ACTION *ret,                  // OUT: return action
    struct PolicyUserInfo *user_info   //OUT this IP's user info
                                                        //if user_info->name[0]='\0', this IP is unauthenticated(but may have langroup)
) ;

// query filename policy
POLICYD_RET_CODE queryFilenamePolicy(
    void *instance,
    const char * filename,
    const char * ip,
    const char * ip2,                            //IN: STM:user's IP, if ip2=NULL, only check *ip; UTM: NULL: non vpn user; others: vpn user
    POLICYD_ACTION *ret,                  // OUT: return action
    struct PolicyUserInfo *user_info,  //OUT this IP's user info
                                                        //if user_info->name[0]='\0', this IP is unauthenticated(but may have langroup)
    unsigned char protocol                  //IN: the type of protocol for this query, every bit is a type of protocol
) ;

// destroy policy client instance
void destroyPolicyClientInstance (void *ins) ;

const char *getPolicydVersion() ;
#ifdef __cplusplus 
}
#endif

#endif

